import { RawOptionsFromRefiners, EventStore, EventUi, Identity, Dictionary, parseClassNames, CalendarContext, DateRange, Action, MountArg, CustomContentGenerator, DidMountHandler, WillUnmountHandler, OrderSpec, DayTableModel, DayTableCell, SlicedCoordRange, SlicedProps, EventRangeProps, EventSegUiInteractionState, Splitter, DateSpan, EventDef, SplittableProps, ViewProps } from '@fullcalendar/core/internal';
import { BusinessHoursInput, ConstraintInput, AllowFunc, EventApi, ViewApi, ClassNamesGenerator } from '@fullcalendar/core';
import { ComponentChild } from '@fullcalendar/core/preact';

declare const RESOURCE_REFINERS: {
    id: StringConstructor;
    parentId: StringConstructor;
    children: Identity<ResourceInput[]>;
    title: StringConstructor;
    businessHours: Identity<BusinessHoursInput>;
    extendedProps: Identity<Dictionary>;
    eventEditable: BooleanConstructor;
    eventStartEditable: BooleanConstructor;
    eventDurationEditable: BooleanConstructor;
    eventConstraint: Identity<ConstraintInput>;
    eventOverlap: BooleanConstructor;
    eventAllow: Identity<AllowFunc>;
    eventClassNames: typeof parseClassNames;
    eventBackgroundColor: StringConstructor;
    eventBorderColor: StringConstructor;
    eventTextColor: StringConstructor;
    eventColor: StringConstructor;
};
type BuiltInResourceRefiners = typeof RESOURCE_REFINERS;
interface ResourceRefiners extends BuiltInResourceRefiners {
}
type ResourceInput = RawOptionsFromRefiners<Required<ResourceRefiners>> & // Required hack
{
    [extendedProps: string]: any;
};
interface Resource {
    id: string;
    parentId: string;
    title: string;
    businessHours: EventStore | null;
    ui: EventUi;
    extendedProps: {
        [extendedProp: string]: any;
    };
}
type ResourceHash = {
    [resourceId: string]: Resource;
};
declare function getPublicId(id: string): string;

declare class ResourceApi {
    private _context;
    _resource: Resource;
    constructor(_context: CalendarContext, _resource: Resource);
    setProp(name: string, value: any): void;
    setExtendedProp(name: string, value: any): void;
    private sync;
    remove(): void;
    getParent(): ResourceApi | null;
    getChildren(): ResourceApi[];
    getEvents(): EventApi[];
    get id(): string;
    get title(): string;
    get eventConstraint(): any;
    get eventOverlap(): boolean;
    get eventAllow(): any;
    get eventBackgroundColor(): string;
    get eventBorderColor(): string;
    get eventTextColor(): string;
    get eventClassNames(): string[];
    get extendedProps(): {
        [extendedProp: string]: any;
    };
    toPlainObject(settings?: {
        collapseExtendedProps?: boolean;
        collapseEventColor?: boolean;
    }): Dictionary;
    toJSON(): Dictionary;
}

interface ResourceSource<ResourceSourceMeta> {
    _raw: any;
    sourceId: string;
    sourceDefId: number;
    meta: ResourceSourceMeta;
    publicId: string;
    isFetching: boolean;
    latestFetchId: string;
    fetchRange: DateRange | null;
}

type ResourceAction = Action | {
    type: 'FETCH_RESOURCE';
} | {
    type: 'RECEIVE_RESOURCES';
    rawResources: ResourceInput[];
    fetchId: string;
    fetchRange: DateRange | null;
} | {
    type: 'RECEIVE_RESOURCE_ERROR';
    error: Error;
    fetchId: string;
    fetchRange: DateRange | null;
} | {
    type: 'ADD_RESOURCE';
    resourceHash: ResourceHash;
} | // use a hash because needs to accept children
{
    type: 'REMOVE_RESOURCE';
    resourceId: string;
} | {
    type: 'SET_RESOURCE_PROP';
    resourceId: string;
    propName: string;
    propValue: any;
} | {
    type: 'SET_RESOURCE_EXTENDED_PROP';
    resourceId: string;
    propName: string;
    propValue: any;
} | {
    type: 'SET_RESOURCE_ENTITY_EXPANDED';
    id: string;
    isExpanded: boolean;
} | {
    type: 'RESET_RESOURCE_SOURCE';
    resourceSourceInput: any;
} | {
    type: 'REFETCH_RESOURCES';
};

type ResourceEntityExpansions = {
    [id: string]: boolean;
};

interface ResourceFuncArg {
    start?: Date;
    end?: Date;
    startStr?: string;
    endStr?: string;
    timeZone?: string;
}
type ResourceFunc = ((arg: ResourceFuncArg, successCallback: (resourceInputs: ResourceInput[]) => void, failureCallback: (error: Error) => void) => void) | ((arg: ResourceFuncArg) => Promise<ResourceInput[]>);

declare const RESOURCE_SOURCE_REFINERS: {
    id: StringConstructor;
    resources: Identity<ResourceFunc | ResourceInput[]>;
    url: StringConstructor;
    method: StringConstructor;
    startParam: StringConstructor;
    endParam: StringConstructor;
    timeZoneParam: StringConstructor;
    extraParams: Identity<Dictionary | (() => Dictionary)>;
};
type ResourceSourceInputObject = RawOptionsFromRefiners<typeof RESOURCE_SOURCE_REFINERS>;
type ResourceSourceInput = ResourceSourceInputObject | ResourceInput[] | ResourceFunc | string;

interface ResourceLabelContentArg {
    resource: ResourceApi;
    text: string;
    isDisabled: boolean;
    date?: Date;
    view: ViewApi;
}
type ResourceLabelMountArg = MountArg<ResourceLabelContentArg>;

interface ColHeaderContentArg {
    view: ViewApi;
}
type ColHeaderMountArg = MountArg<ColHeaderContentArg>;
interface ColCellContentArg {
    resource?: ResourceApi;
    groupValue?: any;
    view: ViewApi;
}
type ColCellMountArg = MountArg<ColCellContentArg>;
interface ColHeaderRenderHooks {
    headerClassNames?: ClassNamesGenerator<ColHeaderContentArg>;
    headerContent?: CustomContentGenerator<ColHeaderContentArg>;
    headerDefault?: (renderProps: ColHeaderContentArg) => ComponentChild;
    headerDidMount?: DidMountHandler<ColHeaderMountArg>;
    headerWillUnmount?: WillUnmountHandler<ColHeaderMountArg>;
}
interface ColSpec extends ColHeaderRenderHooks {
    group?: boolean;
    isMain?: boolean;
    width?: number | string;
    field?: string;
    cellClassNames?: ClassNamesGenerator<ColCellContentArg>;
    cellContent?: CustomContentGenerator<ColCellContentArg>;
    cellDidMount?: DidMountHandler<ColCellMountArg>;
    cellWillUnmount?: WillUnmountHandler<ColCellMountArg>;
}
interface GroupLaneRenderHooks {
    laneClassNames?: ClassNamesGenerator<ColCellContentArg>;
    laneContent?: CustomContentGenerator<ColCellContentArg>;
    laneDidMount?: DidMountHandler<ColCellMountArg>;
    laneWillUnmount?: WillUnmountHandler<ColCellMountArg>;
}
interface GroupSpec extends GroupLaneRenderHooks {
    field?: string;
    order?: number;
    labelClassNames?: ClassNamesGenerator<ColCellContentArg>;
    labelContent?: CustomContentGenerator<ColCellContentArg>;
    labelDidMount?: DidMountHandler<ColCellMountArg>;
    labelWillUnmount?: WillUnmountHandler<ColCellMountArg>;
}

interface ResourceLaneContentArgInput {
    resource: Resource;
    context: CalendarContext;
}
interface ResourceLaneContentArg {
    resource: ResourceApi;
}
type ResourceLaneMountArg = MountArg<ResourceLaneContentArg>;
declare function refineRenderProps(input: ResourceLaneContentArgInput): ResourceLaneContentArg;

declare const DEFAULT_RESOURCE_ORDER: OrderSpec<unknown>[];
interface ResourceAddArg {
    resource: ResourceApi;
    revert: () => void;
}
interface ResourceChangeArg {
    oldResource: ResourceApi;
    resource: ResourceApi;
    revert: () => void;
}
interface ResourceRemoveArg {
    resource: ResourceApi;
    revert: () => void;
}

declare class ResourceIndex {
    indicesById: {
        [resourceId: string]: number;
    };
    ids: string[];
    length: number;
    constructor(resources: Resource[]);
}

declare abstract class AbstractResourceDayTableModel {
    dayTableModel: DayTableModel;
    resources: Resource[];
    rowCnt: number;
    colCnt: number;
    cells: DayTableCell[][];
    resourceIndex: ResourceIndex;
    constructor(dayTableModel: DayTableModel, resources: Resource[], rowCnt: number, colCnt: number);
    abstract computeCol(dateI: number, resourceI: number): number;
    abstract computeColRanges(dateStartI: number, dateEndI: number, resourceI: number): SlicedCoordRange[];
    abstract buildCells(): DayTableCell[][];
}

declare class ResourcelessDayTableModel extends AbstractResourceDayTableModel {
    constructor(dayTableModel: DayTableModel);
    computeCol(dateI: number, _resourceI: number): number;
    computeColRanges(dateStartI: number, dateEndI: number, _resourceI: number): SlicedCoordRange[];
    buildCells(): DayTableCell[][];
}

declare abstract class ResourcefulDayTableModel extends AbstractResourceDayTableModel {
    private context;
    constructor(dayTableModel: DayTableModel, resources: Resource[], context: CalendarContext);
    abstract computeCol(dateI: number, resourceI: number): number;
    abstract computeColRanges(dateStartI: number, dateEndI: number, resourceI: number): SlicedCoordRange[];
    buildCells(): DayTableCell[][];
}

declare class ResourceDayTableModel extends ResourcefulDayTableModel {
    computeCol(dateI: number, resourceI: number): number;
    computeColRanges(dateStartI: number, dateEndI: number, resourceI: number): {
        start: number;
        end: number;
        isStart: boolean;
        isEnd: boolean;
    }[];
}

declare class DayResourceTableModel extends ResourcefulDayTableModel {
    computeCol(dateI: any, resourceI: any): any;
    computeColRanges(dateStartI: number, dateEndI: number, resourceI: number): any[];
}

declare abstract class VResourceJoiner<R> {
    private joinDateSelection;
    private joinBusinessHours;
    private joinFgEvents;
    private joinBgEvents;
    private joinEventDrags;
    private joinEventResizes;
    joinProps(propSets: {
        [resourceId: string]: SlicedProps<R>;
    }, resourceDayTable: AbstractResourceDayTableModel): SlicedProps<R>;
    joinSegs(resourceDayTable: AbstractResourceDayTableModel, ...segGroups: (R & EventRangeProps)[][]): (R & EventRangeProps)[];
    expandSegs(resourceDayTable: AbstractResourceDayTableModel, segs: R[]): (R & EventRangeProps)[];
    joinInteractions(resourceDayTable: AbstractResourceDayTableModel, ...interactions: EventSegUiInteractionState<R>[]): EventSegUiInteractionState<R> | null;
    abstract transformSeg(seg: R & EventRangeProps, resourceDayTable: AbstractResourceDayTableModel, resourceI: number): (R & EventRangeProps)[];
}

interface VResourceProps extends SplittableProps {
    resourceDayTableModel: AbstractResourceDayTableModel;
}
declare class VResourceSplitter extends Splitter<VResourceProps> {
    getKeyInfo(props: VResourceProps): any;
    getKeysForDateSpan(dateSpan: DateSpan): string[];
    getKeysForEventDef(eventDef: EventDef): string[];
}

interface ResourceViewProps extends ViewProps {
    resourceStore: ResourceHash;
    resourceEntityExpansions: ResourceEntityExpansions;
}

interface Group {
    spec: GroupSpec;
    value: any;
}
interface GenericNode {
    entity: Resource | Group;
    children: GenericNode[];
}
interface GroupNode extends GenericNode {
    entity: Group;
    pooledHeight: boolean;
    children: (GroupNode | ResourceNode)[];
}
interface ResourceNode extends GenericNode {
    entity: Resource;
    resourceFields: any;
    children: ResourceNode[];
}
type ResourceNodeHash = {
    [resourceId: string]: ResourceNode;
};
declare function buildResourceHierarchy(resourceStore: ResourceHash, orderSpecs: OrderSpec<ResourceApi>[], // why accepted when resourceNodeHash already ordered?
groupSpecs?: GroupSpec[], groupRowDepth?: number): GenericNode[];
declare function flattenResources(resourceStore: ResourceHash, orderSpecs: OrderSpec<ResourceApi>[]): Resource[];
declare function createEntityId(entity: Resource | Group): string;
declare function createGroupId(group: Group): string;
declare function isEntityGroup(entity: Resource | Group): entity is Group;
declare function isGroupsEqual(group0: Group, group1: Group): boolean;

interface SplittableResourceProps extends SplittableProps {
    resourceStore: ResourceHash;
}
declare class ResourceSplitter extends Splitter<SplittableResourceProps> {
    getKeyInfo(props: SplittableResourceProps): {
        '': {};
    };
    getKeysForDateSpan(dateSpan: DateSpan): string[];
    getKeysForEventDef(eventDef: EventDef): string[];
}

export { AbstractResourceDayTableModel as A, DayResourceTableModel as B, ColSpec as C, DEFAULT_RESOURCE_ORDER as D, VResourceSplitter as E, Resource as F, GroupLaneRenderHooks as G, getPublicId as H, ResourceViewProps as I, Group as J, GenericNode as K, buildResourceHierarchy as L, ResourceNodeHash as M, isEntityGroup as N, createGroupId as O, isGroupsEqual as P, flattenResources as Q, ResourceSourceInput as R, createEntityId as S, ResourceNode as T, GroupNode as U, VResourceJoiner as V, GroupSpec as W, ResourceSplitter as X, ColHeaderContentArg as a, ColHeaderMountArg as b, ColCellContentArg as c, ColCellMountArg as d, ResourceLabelContentArg as e, ResourceLabelMountArg as f, ResourceLaneContentArg as g, ResourceLaneMountArg as h, ResourceApi as i, ResourceAddArg as j, ResourceChangeArg as k, ResourceRemoveArg as l, ResourceAction as m, ResourceSource as n, ResourceHash as o, ResourceEntityExpansions as p, ResourceInput as q, ResourceLaneContentArgInput as r, ColHeaderRenderHooks as s, ResourceFunc as t, ResourceFuncArg as u, ResourceSourceInputObject as v, refineRenderProps as w, ResourcelessDayTableModel as x, ResourcefulDayTableModel as y, ResourceDayTableModel as z };
